@@ -3,7 +3,7 @@
 import math
 from decimal import Decimal
 from fractions import Fraction
-import itertools
+from typing import Dict, Set
 
 from ss2hcsp.util.topsort import topological_sort
 
@@ -51,26 +51,30 @@ class Expr:
     def __init__(self):
         pass
 
-    def priority(self):
+    def priority(self) -> int:
         """Returns priority during printing."""
         raise NotImplementedError
 
-    def get_vars(self):
+    def get_vars(self) -> Set[str]:
         """Returns set of variables in the expression."""
         raise NotImplementedError
 
-    def get_fun_names(self):
+    def get_fun_names(self) -> Set[str]:
         """Return set of function names in the expression"""
         return NotImplementedError
 
-    def get_zero_arity_funcs(self):
+    def get_zero_arity_funcs(self) -> Set[str]:
         """Return set of functions with zero arity in the expression"""
         return NotImplementedError
 
-    def subst(self, inst):
+    def subst(self, inst: Dict[str, "Expr"]):
         """inst is a dictionary mapping variable names to expressions."""
         raise NotImplementedError
 
+    def simplify(self) -> "Expr":
+        """Return a simplified version of the expression."""
+        raise NotImplementedError
+
 
 class AVar(Expr):
     def __init__(self, name, meta=None):
@@ -109,13 +113,14 @@ class AVar(Expr):
         else:
             return self
 
+    def simplify(self) -> Expr:
+        return self
+
 
 class AConst(Expr):
     def __init__(self, value, meta=None):
         super(AConst, self).__init__()
-        assert isinstance(value, (int, float, Decimal, Fraction, list, str, tuple, dict))
-        # if isinstance(value, Decimal):
-        #     value = float(value)
+        assert isinstance(value, (int, float, Decimal, Fraction, str))
         self.value = value
         self.meta = meta
 
@@ -128,6 +133,9 @@ class AConst(Expr):
     def __eq__(self, other):
         return isinstance(other, AConst) and self.value == other.value
 
+    def __lt__(self, other):
+        return self.value < other.value
+
     def __hash__(self):
         return hash(("AConst", str(self.value)))
 
@@ -146,6 +154,9 @@ class AConst(Expr):
     def subst(self, inst):
         return self
 
+    def simplify(self) -> Expr:
+        return self
+
 
 class OpExpr(Expr):
     def __init__(self, op, *exprs, meta=None):
@@ -212,6 +223,10 @@ class OpExpr(Expr):
     def subst(self, inst):
         return OpExpr(self.op, *(expr.subst(inst) for expr in self.exprs))
 
+    def simplify(self) -> Expr:
+        return OpExpr(self.op, *(expr.simplify() for expr in self.exprs))
+
+
 def list_add(*args):
     if len(args) == 0:
         return 0
@@ -242,7 +257,10 @@ class FunExpr(Expr):
         return "Fun(%s, %s)" % (self.fun_name, ", ".join(repr(expr) for expr in self.exprs))
 
     def __str__(self):
-        return "%s(%s)" % (self.fun_name, ",".join(str(expr) for expr in self.exprs))
+        if self.exprs:
+            return "%s(%s)" % (self.fun_name, ",".join(str(expr) for expr in self.exprs))
+        else:
+            return self.fun_name
 
     def __eq__(self, other):
         return isinstance(other, FunExpr) and self.fun_name == other.fun_name and \
@@ -271,6 +289,10 @@ class FunExpr(Expr):
     def subst(self, inst):
         return FunExpr(self.fun_name, [expr.subst(inst) for expr in self.exprs])
 
+    def simplify(self) -> Expr:
+        return FunExpr(self.fun_name, [expr.simplify() for expr in self.exprs])
+
+
 def replace_function(e: FunExpr, funcs=dict()):
     """Replace a FunExpr by the expr of its corresponding Function object.
     For example,
@@ -360,6 +382,9 @@ class IfExpr(Expr):
     def subst(self, inst):
         return IfExpr(self.cond.subst(inst), self.expr1.subst(inst), self.expr2.subst(inst))
 
+    def simplify(self) -> Expr:
+        return IfExpr(self.cond.simplify(), self.expr1.simplify(), self.expr2.simplify())
+
 
 class ListExpr(Expr):
     """List expressions."""
@@ -398,6 +423,9 @@ class ListExpr(Expr):
     def subst(self, inst):
         return ListExpr(*(expr.subst(inst) for expr in self.args))
 
+    def simplify(self) -> Expr:
+        return ListExpr(*(expr.simplify() for expr in self.args))
+
 
 class DictExpr(Expr):
     """Dictionary expressions (structures)."""
@@ -438,6 +466,9 @@ class DictExpr(Expr):
     def subst(self, inst):
         return DictExpr(*((k, v.subst(inst)) for k, v in self.dict.items()))
 
+    def simplify(self) -> Expr:
+        return DictExpr(*((k, v.simplify()) for k, v in self.dict.items()))
+
 
 class ArrayIdxExpr(Expr):
     """Expressions of the form a[i], where a evaluates to a list and i
@@ -490,6 +521,13 @@ class ArrayIdxExpr(Expr):
     def subst(self, inst):
         return ArrayIdxExpr(expr1=self.expr1.subst(inst), expr2=self.expr2.subst(inst))
 
+    def simplify(self) -> Expr:
+        e1, e2 = self.expr1.simplify(), self.expr2.simplify()
+        if isinstance(self.expr1, ListExpr) and isinstance(self.expr2, AConst):
+            return self.expr1.args[int(self.expr2.value)]
+        else:
+            return ArrayIdxExpr(e1, e2)
+
 
 class FieldNameExpr(Expr):
     """Expression of the form a.name, where a evaluates to a structure
@@ -531,6 +569,10 @@ class FieldNameExpr(Expr):
     def subst(self, inst):
         return FieldNameExpr(expr=self.expr.subst(inst), field=self.field)
 
+    def simplify(self) -> Expr:
+        return FieldNameExpr(self.expr.simplify(), self.field)
+
+
 class BConst(Expr):  # Boolean expression
     def __init__(self, value, meta=None):
         super(BConst, self).__init__()
@@ -565,6 +607,10 @@ class BConst(Expr):  # Boolean expression
     def subst(self, inst):
         return self
 
+    def simplify(self) -> Expr:
+        return self
+
+
 true_expr = BConst(True)
 false_expr = BConst(False)
 
@@ -632,6 +678,9 @@ class LogicExpr(Expr):
     def subst(self, inst):
         return LogicExpr(self.op, *(expr.subst(inst) for expr in self.exprs))
 
+    def simplify(self) -> Expr:
+        return LogicExpr(self.op, *(expr.simplify() for expr in self.exprs))
+
 
 def list_conj(*args):
     if len(args) == 0:
@@ -742,6 +791,10 @@ class RelExpr(Expr):
     def subst(self, inst):
         return RelExpr(self.op, self.expr1.subst(inst), self.expr2.subst(inst))
 
+    def simplify(self) -> Expr:
+        return RelExpr(self.op, self.expr1.simplify(), self.expr2.simplify())
+
+
 class ExistsExpr(Expr):
     """Exists expressions"""
     def __init__(self, vars, expr, meta=None):
@@ -794,6 +847,10 @@ class ExistsExpr(Expr):
             assert str(var) not in inst
         return ExistsExpr(self.vars, self.expr.subst(inst))
 
+    def simplify(self) -> Expr:
+        return ExistsExpr(self.vars, self.expr.simplify())
+
+
 class ForAllExpr(Expr):
     """ForAll expressions"""
     def __init__(self, vars, expr, meta=None):
@@ -846,6 +903,10 @@ class ForAllExpr(Expr):
             assert str(var) not in inst
         return ForAllExpr(self.vars, self.expr.subst(inst))
 
+    def simplify(self) -> Expr:
+        return ForAllExpr(self.vars, self.expr.simplify())
+
+
 def neg_expr(e):
     """Returns the negation of an expression, using deMorgan's law to move
     negation inside.
@@ -897,87 +958,3 @@ def subst_all(e, inst):
     for var in reversed(topo_order):
         e = e.subst({var: inst[var]})
     return e
-
-
-class Conditional_Inst:
-    """A mapping from variables to lists of conditional substitutions.
-
-    For example:
-    {
-        x: [(True, y + 1)],
-        z: [(x >= 1, y + 1), (x < 1, y - 1)]
-    }
-    """
-    def __init__(self):
-        self.data = dict()
-        self.mu_ex_conds = list()  # record a list of sets of mutually exclusive constraints
-
-    def __repr__(self):
-        def repr_pair(cond, inst):
-            return "(%s, %s)" % (str(cond), str(inst))
-
-        def repr_cond_inst(cond_inst):
-            return "[" + ", ".join(repr_pair(cond, inst) for cond, inst in cond_inst) + "]"
-
-        return "\n".join(var + ": " + repr_cond_inst(cond_inst) for var, cond_inst in self.data.items())
-
-    def conflicting(self, conditions):
-        assert isinstance(conditions, set)
-        if len(conditions) == 0:
-            return False  # No conflict
-        elif len(conditions) == 1:
-            condition = list(conditions)[0]
-            if condition == false_expr:
-                return True  # Conflict
-            else:
-                return False  # No conflict
-
-        for cond0, cond1 in itertools.combinations_with_replacement(conditions, 2):
-            if cond0 != cond1:
-                for mu_ex_cond_set in self.mu_ex_conds:
-                    if {cond0, cond1}.issubset(mu_ex_cond_set):
-                        return True
-        return False
-
-    def add(self, var_name, cond_inst):
-        """Add a new instantiation."""
-        assert var_name not in self.data
-        
-        # Substitute using existing instantiations
-        for var in self.data:
-            new_cond_inst = []
-            for cond, expr in cond_inst:
-                if var in cond.get_vars() or var in expr.get_vars():
-                    for cond_var, expr_var in self.data[var]:
-                        new_cond = cond.subst({var: expr_var})
-                        if self.conflicting({new_cond, cond_var}) or conj(new_cond, cond_var) == false_expr:
-                            continue  # because new_cond & cond_var is False
-                        else:
-                            new_cond = conj(new_cond, cond_var)
-                        new_expr = expr.subst({var: expr_var})
-                        new_cond_inst.append((new_cond, new_expr))
-                else:
-                    new_cond_inst.append((cond, expr))
-            cond_inst = new_cond_inst
-        # Check: the conditons in cond_inst are different
-        conditions = [cond for cond, _ in cond_inst]
-        assert len(conditions) == len(set(conditions))
-
-        # Merge (cond, expr) pairs with the same expression
-        expr_dict = dict()
-        for cond, expr in cond_inst:
-            if expr not in expr_dict:
-                expr_dict[expr] = []
-            expr_dict[expr].append(cond)
-
-        cond_inst = []
-        for expr, conds in sorted(expr_dict.items(), key=str):
-            cond_inst.append((disj(*conds), expr))
-
-        self.data[var_name] = cond_inst
-
-        # Extract mutually exclusive constraints
-        if len(cond_inst) >= 2:
-            mu_ex_cons = set(cond for cond, _ in cond_inst)
-            if mu_ex_cons not in self.mu_ex_conds:
-                self.mu_ex_conds.append(mu_ex_cons)
