@@ -1,7 +1,7 @@
 """Hybrid programs"""
 
 from collections import OrderedDict
-from typing import List, Union
+from typing import Dict, List, Optional, Tuple, Union, Set
 
 from ss2hcsp.hcsp.expr import Expr, AVar, AConst, Expr, true_expr, false_expr, RelExpr, LogicExpr, Expr
 from ss2hcsp.hcsp import assertion
@@ -19,27 +19,19 @@ class Channel:
     to be matched are indicated by variables.
 
     """
-    def __init__(self, name, args=None, meta=None):
+    def __init__(self, name: str, args: Optional[Union[int, str, Expr]] = None, meta=None):
         assert isinstance(name, str)
-        if args is None:
-            args = tuple()
-
-        # Each argument is either integer, (unevaluated) expression, or variable
-        assert isinstance(args, tuple) and all(isinstance(arg, (Expr, int, str)) for arg in args)
-
-        self.name = name
-        self.args = args
+        if args:
+            assert all(isinstance(arg, (int, str, Expr)) for arg in args)
+        self.name: str = name
+        self.args: tuple[Expr] = tuple()
+        if args is not None:
+            self.args = tuple([AConst(arg) if isinstance(arg, (int, str)) else arg for arg in args])
         self.meta = meta
 
     def __str__(self):
-        def str_arg(arg):
-            # if isinstance(arg, str):
-            #     return repr(arg)
-            if isinstance(arg, AConst) and isinstance(arg.value, str) and arg.value[0] != "\"":
-                return "\"" + arg.value + "\""
-            else:
-                return str(arg)
-        return self.name + ''.join('[' + str_arg(arg) + ']' for arg in self.args)
+        return self.name + ''.join('[' + str(arg) + ']' for arg in self.args)
+        # return self.name + ''.join('_l_' + str(arg) + '_r_' for arg in self.args)
     
     def __repr__(self):
         if self.args:
@@ -59,15 +51,13 @@ class Channel:
     def __lt__(self, other):
         return (self.name, self.args) < (other.name, other.args)
 
-    def subst(self, inst):
+    def subst(self, inst) -> "Channel":
         if self.name in inst:
             target = inst[self.name]
             assert isinstance(target, Channel)
-            return Channel(target.name, tuple(arg if isinstance(arg, (int, str)) else arg.subst(inst)
-                                              for arg in target.args + self.args))
+            return Channel(target.name, tuple(arg.subst(inst) for arg in target.args + self.args))
         else:
-            return Channel(self.name, tuple(arg if isinstance(arg, (int, str)) else arg.subst(inst)
-                                            for arg in self.args))
+            return Channel(self.name, tuple(arg.subst(inst) for arg in self.args))
 
 class Function:
     """Declarations of (pure functions).
@@ -93,6 +83,7 @@ class Function:
         return "Function(%s, %s, %s)" % (self.name, repr(self.vars), repr(self.expr))
 
 
+
 class HCSP:
     def __init__(self):
         self.type = None
@@ -116,7 +107,7 @@ class HCSP:
     def get_chs(self):
         return set()
 
-    def priority(self):
+    def priority(self) -> int:
         if self.type == "parallel":
             return 30
         elif self.type == "select_comm":
@@ -128,7 +119,7 @@ class HCSP:
         else:
             return 100
 
-    def size(self):
+    def size(self) -> int:
         """Returns size of HCSP program."""
         if isinstance(self, (Var, Skip, Wait, Assign, Assert, Test, Log,
                              InputChannel, OutputChannel)):
@@ -140,14 +131,14 @@ class HCSP:
         elif isinstance(self, ODE):
             return 1
         elif isinstance(self, (ODE_Comm, SelectComm)):
-            return 1 + sum([1 + out_hp.size() for comm_hp, out_hp in self.io_comms])
+            return 1 + sum([1 + out_hp.size() for _, out_hp in self.io_comms])
         elif isinstance(self, ITE):
-            return 1 + sum([1 + hp.size() for cond, hp in self.if_hps]) + \
-              (self.else_hp.size() if self.else_hp else 0)
+            return 1 + sum([1 + hp.size() for _, hp in self.if_hps]) + \
+                       (self.else_hp.size() if self.else_hp else 0)
         else:
             raise NotImplementedError
 
-    def contain_hp(self, name):
+    def contain_hp(self, name) -> bool:
         """Returns whether the given HCSP program contains Var(name)."""
         if isinstance(self, Var):
             return self.name == name
@@ -174,7 +165,7 @@ class HCSP:
         else:
             raise NotImplementedError
 
-    def get_contain_hps_count(self):
+    def get_contain_hps_count(self) -> Dict[str, int]:
         """Returns the dictionary mapping contained hps to number
         of appearances.
         
@@ -208,53 +199,53 @@ class HCSP:
         rec(self)
         return res
 
-    def get_contain_hps(self):
+    def get_contain_hps(self) -> Set[str]:
         """Returns the set of Var calls contained in self."""
         return set(self.get_contain_hps_count())
 
     def subst_comm(self, inst):
-        def subst_io_comm(io_comm):
+        def subst_io_comm(io_comm: Tuple[HCSP, HCSP]):
             return (io_comm[0].subst_comm(inst), io_comm[1].subst_comm(inst))
 
-        def subst_if_hp(if_hp):
+        def subst_if_hp(if_hp: Tuple[Expr, HCSP]):
             return (if_hp[0].subst(inst), if_hp[1].subst_comm(inst))
 
-        def subst_ode_eq(ode_eq):
+        def subst_ode_eq(ode_eq: Tuple[str, Expr]):
             return (ode_eq[0], ode_eq[1].subst(inst))
 
-        if self.type in ('var', 'skip'):
+        if isinstance(self, (Var, Skip)):
             return self
-        elif self.type == 'wait':
+        elif isinstance(self, Wait):
             return Wait(self.delay.subst(inst))
-        elif self.type == 'assign':
+        elif isinstance(self, Assign):
             return Assign(self.var_name, self.expr.subst(inst))
-        elif self.type == 'assert':
+        elif isinstance(self, Assert):
             return Assert(self.bexpr.subst(inst), [expr.subst(inst) for expr in self.msgs])
-        elif self.type == 'test':
+        elif isinstance(self, Test):
             return Test(self.bexpr.subst(inst), [expr.subst(inst) for expr in self.msgs])
-        elif self.type == 'log':
+        elif isinstance(self, Log):
             return Log(self.pattern.subst(inst), exprs=[expr.subst(inst) for expr in self.exprs])
-        elif self.type == 'input_channel':
+        elif isinstance(self, InputChannel):
             return InputChannel(self.ch_name.subst(inst), self.var_name)
-        elif self.type == 'output_channel':
+        elif isinstance(self, OutputChannel):
             if self.expr is None:
                 return OutputChannel(self.ch_name.subst(inst))
             else:
                 return OutputChannel(self.ch_name.subst(inst), self.expr.subst(inst))
-        elif self.type == 'sequence':
+        elif isinstance(self, Sequence):
             return Sequence(*(hp.subst_comm(inst) for hp in self.hps))
-        elif self.type == 'ode':
+        elif isinstance(self, ODE):
             return ODE([subst_ode_eq(eq) for eq in self.eqs],
                        self.constraint.subst(inst), out_hp=self.out_hp.subst_comm(inst))
-        elif self.type == 'ode_comm':
+        elif isinstance(self, ODE_Comm):
             return ODE_Comm([subst_ode_eq(eq) for eq in self.eqs],
                             self.constraint.subst(inst),
                             [subst_io_comm(io_comm) for io_comm in self.io_comms])
-        elif self.type == 'loop':
+        elif isinstance(self, Loop):
             return Loop(self.hp.subst_comm(inst), constraint=self.constraint)
-        elif self.type == 'select_comm':
+        elif isinstance(self, SelectComm):
             return SelectComm(*(subst_io_comm(io_comm) for io_comm in self.io_comms))
-        elif self.type == 'ite':
+        elif isinstance(self, ITE):
             return ITE([subst_if_hp(if_hp) for if_hp in self.if_hps], 
               self.else_hp.subst_comm(inst) if self.else_hp is not None else None)
         else:
@@ -263,7 +254,7 @@ class HCSP:
 
 
 class Var(HCSP):
-    def __init__(self, name, meta=None):
+    def __init__(self, name: str, meta=None):
         super(Var, self).__init__()
         self.type = "var" 
         assert isinstance(name, str)
@@ -459,10 +450,10 @@ class Assert(HCSP):
         super(Assert, self).__init__()
         self.type = "assert"
         assert isinstance(bexpr, Expr)
-        self.bexpr = bexpr
+        self.bexpr: Expr = bexpr
         msgs = tuple(msgs)
         assert all(isinstance(msg, Expr) for msg in msgs)
-        self.msgs = msgs
+        self.msgs: Tuple[Expr] = msgs
         self.meta = meta
 
     def __eq__(self, other):
@@ -507,10 +498,10 @@ class Test(HCSP):
         super(Test, self).__init__()
         self.type = "test"
         assert isinstance(bexpr, Expr)
-        self.bexpr = bexpr
+        self.bexpr: Expr = bexpr
         msgs = tuple(msgs)
         assert all(isinstance(msg, Expr) for msg in msgs)
-        self.msgs = msgs
+        self.msgs: Tuple[Expr] = msgs
         self.meta = meta
 
     def __eq__(self, other):
@@ -551,17 +542,17 @@ class Test(HCSP):
 
 
 class Log(HCSP):
-    def __init__(self, pattern, *, exprs=None, meta=None):
+    def __init__(self, pattern: Expr, *, exprs=None, meta=None):
         super(Log, self).__init__()
         self.type = "log"
         assert isinstance(pattern, Expr)
-        self.pattern = pattern
+        self.pattern: Expr = pattern
         if exprs is None:
             exprs = tuple()
         else:
             exprs = tuple(exprs)
         assert all(isinstance(expr, Expr) for expr in exprs)
-        self.exprs = exprs
+        self.exprs: Tuple[Expr] = exprs
         self.meta = meta
 
     def __eq__(self, other):
@@ -601,17 +592,17 @@ class Log(HCSP):
         return fun_set
     
 class InputChannel(HCSP):
-    def __init__(self, ch_name, var_name=None, meta=None):
+    def __init__(self, ch_name: Union[str, Channel], var_name: Optional[Union[str, Expr]] = None, meta=None):
         super(InputChannel, self).__init__()
         self.type = "input_channel"
         if isinstance(ch_name, str):
             ch_name = Channel(ch_name)
         assert isinstance(ch_name, Channel)
         if isinstance(var_name, str):
-            var_name = AVar(str(var_name))
+            var_name = AVar(var_name)
         assert var_name is None or isinstance(var_name, Expr)
-        self.ch_name = ch_name  # Channel
-        self.var_name = var_name  # Expr or None
+        self.ch_name: Channel = ch_name
+        self.var_name: Optional[Expr] = var_name
         self.meta = meta
 
     def __eq__(self, other):
@@ -673,15 +664,15 @@ class ParaInputChannel(InputChannel):
 
 
 class OutputChannel(HCSP):
-    def __init__(self, ch_name, expr=None, meta=None):
+    def __init__(self, ch_name: Union[str, Channel], expr: Optional[Expr] = None, meta=None):
         super(OutputChannel, self).__init__()
         self.type = "output_channel"
         if isinstance(ch_name, str):
             ch_name = Channel(ch_name)
         assert isinstance(ch_name, Channel)
         assert expr is None or isinstance(expr, Expr)
-        self.ch_name = ch_name  # Channel
-        self.expr = expr  # Expr or None
+        self.ch_name: Channel = ch_name
+        self.expr: Optional[Expr] = expr
         self.meta = meta
 
     def __eq__(self, other):
@@ -738,13 +729,14 @@ class ParaOutputChannel(OutputChannel):
                 result += "[\"" + str(para) + "\"]"
             else:
                 result += "[" + str(para) + "]"
+        result += "!"
         if self.expr:
             result += str(self.expr)
         result += ";"
         return result
 
 
-def is_comm_channel(hp):
+def is_comm_channel(hp: HCSP) -> bool:
     return hp.type == "input_channel" or hp.type == "output_channel"
 
 
@@ -755,7 +747,7 @@ class Sequence(HCSP):
         self.type = "sequence"
         assert all(isinstance(hp, HCSP) for hp in hps)
         assert len(hps) >= 1
-        self.hps = []
+        self.hps: List[HCSP] = []
         for hp in hps:
             if isinstance(hp, Sequence):
                 self.hps.extend(hp.hps)
@@ -981,9 +973,9 @@ class ODE_Comm(HCSP):
             assert is_comm_channel(io_comm[0]) and isinstance(io_comm[1], HCSP)
 
         self.type = "ode_comm"
-        self.eqs = eqs  # list
-        self.constraint = constraint  # Expr
-        self.io_comms = io_comms  # list
+        self.eqs: List[Tuple[str, Expr]] = eqs
+        self.constraint: Expr = constraint
+        self.io_comms: List[Tuple[HCSP, HCSP]] = io_comms
         self.meta = meta
 
     def __eq__(self, other):
@@ -1175,13 +1167,11 @@ class SelectComm(HCSP):
         any program.
         
         """
-        super(SelectComm, self).__init__()
         self.type = "select_comm"
         assert len(io_comms) >= 2
-
         assert all(is_comm_channel(comm_hp) and isinstance(out_hp, HCSP)
                    for comm_hp, out_hp in io_comms)
-        self.io_comms = tuple(io_comms)
+        self.io_comms: List[Tuple[HCSP, HCSP]] = io_comms
         self.meta = meta
 
     def __eq__(self, other):
@@ -1249,8 +1239,8 @@ class ITE(HCSP):
         assert len(if_hps) > 0, "ITE: must have at least one if branch"
         assert else_hp is None or isinstance(else_hp, HCSP)
         self.type = "ite"
-        self.if_hps = tuple(tuple(p) for p in if_hps)
-        self.else_hp = else_hp
+        self.if_hps: List[Tuple[Expr, HCSP]] = list(tuple(p) for p in if_hps)
+        self.else_hp: Optional[HCSP] = else_hp
         self.meta = meta
 
     def __eq__(self, other):
@@ -1315,7 +1305,7 @@ class ITE(HCSP):
             ch_set.update(self.else_hp.get_chs())
         return ch_set
 
-def ite(if_hps, else_hp):
+def ite(if_hps: List[Tuple[Expr, HCSP]], else_hp: Optional[HCSP]) -> HCSP:
     """Construction of if-then-else with simplifications."""
     new_if_hps, new_else_hp = [], else_hp
     for cond, if_hp in if_hps:
@@ -1333,7 +1323,7 @@ def ite(if_hps, else_hp):
     else:
         return ITE(new_if_hps, new_else_hp)
 
-def get_comm_chs(hp):
+def get_comm_chs(hp: HCSP):
     """Returns the list of communication channels for the given program.
     
     Result is a list of pairs (ch_name, '?'/'!').
@@ -1368,13 +1358,32 @@ def get_comm_chs(hp):
     rec(hp)
     return list(OrderedDict.fromkeys(collect))
 
+class Constants:
+    """Represent constants in a program.
+
+    vars: a set of constant vars
+    preds: a list of constant predicates
+    
+    """
+    def __init__(self, vars=set(), preds=list()):
+        assert isinstance(vars, set)
+        assert isinstance(preds, list)
+        self.vars: Set[str] = vars
+        self.preds: List[Expr] = preds
+
+    def __str__(self):
+        return "constants " + ";".join([str(pred) for pred in self.preds])
+
+
 class HoareTriple:
     """A program with pre- and post-conditions"""
-    def __init__(self, pre, hp, post, functions=None, meta=None):
+    def __init__(self, pre, hp, post, constants=Constants(), functions=None, meta=None):
         self.pre = list(pre)
         self.post = list(post)
         assert all(isinstance(subpost, assertion.OrdinaryAssertion) for subpost in post)
         self.hp = hp
+        assert isinstance(constants, Constants)
+        self.constants = constants
         if functions is None:
             functions = dict()
         self.functions = functions
@@ -1382,12 +1391,12 @@ class HoareTriple:
 
 class Procedure:
     """Declared procedure within a process."""
-    def __init__(self, name, hp, meta=None):
-        self.name = name
+    def __init__(self, name: str, hp: Union[str, HCSP], meta=None):
+        self.name: str = name
         if isinstance(hp, str):
             from ss2hcsp.hcsp.parser import hp_parser
             hp = hp_parser.parse(hp)
-        self.hp = hp
+        self.hp: HCSP = hp
         self.meta = meta
 
     def __eq__(self, other):
@@ -1400,14 +1409,44 @@ class Procedure:
         return "procedure %s begin %s end" % (self.name, str(self.hp))
 
 
+class HCSPOutput:
+    """Represents an output info"""
+    def __init__(self, varname: str, expr: Optional[Expr] = None):
+        self.varname: str = varname
+        self.expr: Optional[Expr] = expr
+
+    def __str__(self):
+        if self.expr is None:
+            return self.varname
+        else:
+            return self.varname + " = " + str(self.expr)
+
+    def toJson(self):
+        if self.expr is None:
+            return self.varname
+        else:
+            return [self.varname, str(self.expr)]
+
+def outputFromJson(data) -> HCSPOutput:
+    if isinstance(data, str):
+        return HCSPOutput(data)
+    else:
+        from ss2hcsp.hcsp.parser import expr_parser
+        return HCSPOutput(data[0], expr_parser.parse(data[1]))
+
+
 class HCSPInfo:
     """HCSP process with extra information."""
-    def __init__(self, name, hp, *, outputs=None, procedures=None):
-        self.name = name
-        self.hp = hp
+    def __init__(self, name: str, hp: HCSP, *, outputs: Optional[List[HCSPOutput]]=None,
+                 procedures: Optional[List[Procedure]]=None):
+        self.name: str = name
+        self.hp: HCSP = hp
         
-        # List of output variables, None indicates output everything
-        self.outputs = outputs
+        # List of output variables
+        self.outputs: List[HCSPOutput] = []
+        if outputs is not None:
+            assert all(isinstance(output, HCSPOutput) for output in outputs)
+            self.outputs = tuple(outputs)
 
         # List of declared procedures
         self.procedures: List[Procedure]
@@ -1428,7 +1467,8 @@ class HCSPInfo:
     def __eq__(self, other):
         return self.name == other.name and self.hp == other.hp
 
-def subst_comm_all(hp, inst):
+
+def subst_comm_all(hp: HCSP, inst):
     """Perform all substitutions given in inst. Detect cycles.
     
     First compute a topological sort of dependency in inst, which will
@@ -1448,7 +1488,22 @@ def subst_comm_all(hp, inst):
         hp = hp.subst_comm({var: inst[var]})
     return hp
 
-def HCSP_subst(hp, inst):
+def subst_all(e: Expr, inst):
+    """Perform all substitutions on an expression. Detect cycles."""
+    # Set of all variables to be substituted
+    all_vars = set(inst.keys())
+
+    # Mapping variable to its dependencies
+    dep_order = dict()
+    for var in all_vars:
+        dep_order[var] = list(inst[var].get_vars().intersection(all_vars))
+
+    topo_order = topological_sort(dep_order)
+    for var in reversed(topo_order):
+        e = e.subst({var: inst[var]})
+    return e
+
+def HCSP_subst(hp: HCSP, inst: Dict[str, HCSP]) -> HCSP:
     """Substitution of program variables for their instantiations."""
     assert isinstance(hp, HCSP)
     if isinstance(hp, Var):
@@ -1479,7 +1534,270 @@ def HCSP_subst(hp, inst):
         print(hp)
         raise NotImplementedError
 
-def reduce_procedures(hp, procs, strict_protect=None):
+def get_concrete_chs(infos: List[HCSPInfo]) -> Dict[str, Set[Tuple[Expr]]]:
+    """Return a mapping from channel names to list of concrete instantiations
+    of the channel.
+    
+    """
+    res: Dict[str, Set[Tuple[Expr]]] = dict()
+
+    def rec(hp):
+        if isinstance(hp, (InputChannel, OutputChannel)):
+            if hp.ch_name.name not in res:
+                res[hp.ch_name.name] = set()
+            if all(isinstance(arg, AConst) for arg in hp.ch_name.args):
+                res[hp.ch_name.name].add(hp.ch_name.args)
+        elif isinstance(hp, (Var, Skip, Wait, Assign, Assert, Test, Log)):
+            pass
+        elif isinstance(hp, Loop):
+            rec(hp.hp)
+        elif isinstance(hp, Sequence):
+            for sub_hp in hp.hps:
+                rec(sub_hp)
+        elif isinstance(hp, ODE):
+            rec(hp.out_hp)
+        elif isinstance(hp, (ODE_Comm, SelectComm)):
+            for comm, out_hp in hp.io_comms:
+                rec(comm)
+                rec(out_hp)
+        elif isinstance(hp, ITE):
+            for _, if_hp in hp.if_hps:
+                rec(if_hp)
+            if hp.else_hp is not None:
+                rec(hp.else_hp)
+        else:
+            print(hp)
+            raise NotImplementedError
+
+    for info in infos:
+        rec(info.hp)
+        for proc in info.procedures:
+            rec(proc.hp)
+
+    return res
+
+def convert_to_concrete_chs(hp: HCSP, concrete_map: Dict[str, Set[Tuple[Expr]]]) -> HCSP:
+    """Using the mapping from channel names to concrete instantiations,
+    convert parameterized HCSP program to unparameterized form.
+    
+    """
+    def find_match(ch_name: Channel):
+        res = []
+        for concrete_args in concrete_map[ch_name.name]:
+            if ch_name.args[:-1] == concrete_args[:-1]:
+                res.append(concrete_args[-1])
+        return res
+
+    def rec(hp):
+        if isinstance(hp, InputChannel):
+            assert hp.ch_name.name in concrete_map
+            if len(hp.ch_name.args) == 0:
+                return hp
+            
+            # Currently only handle the case where all but the last index
+            # are constants.
+            assert all(isinstance(arg, AConst) for arg in hp.ch_name.args[:-1])
+
+            last_arg = hp.ch_name.args[-1]
+            new_name = hp.ch_name.name
+            for arg in hp.ch_name.args[:-1]:
+                if isinstance(arg.value, str):
+                    new_name = new_name + '_lb_' + arg.value + '_rb_'
+                else:
+                    new_name = new_name + '_l_' + str(arg) + '_r_'
+                    
+            if isinstance(last_arg, AConst):
+                if isinstance(last_arg.value, str):
+                    new_name = new_name + '_lb_' + last_arg.value + '_rb_'
+                else:
+                    new_name = new_name + '_l_' + str(last_arg) + '_r_'
+                new_args = ()
+                new_hp = InputChannel(Channel(new_name, new_args), hp.var_name)
+                return new_hp
+            elif isinstance(last_arg, AVar) and last_arg.name.startswith("_"):
+                raise NotImplementedError
+            else:
+                # Expression case
+                matches = find_match(hp.ch_name)
+                assert len(matches) > 0
+                if_hps = []
+                for match in sorted(matches):
+                    new_args = ()
+                    if isinstance(match.value, str):
+                        match_name = new_name + "_lb_" + match.value + "_rb_"
+                    else:
+                        match_name = new_name + "_l_" + str(match) + "_r_"
+                    if_hps.append((RelExpr("==", last_arg, match),
+                                   InputChannel(Channel(match_name, new_args), hp.var_name)))
+                return ITE(if_hps)
+
+        elif isinstance(hp, OutputChannel):
+            assert hp.ch_name.name in concrete_map
+            if len(hp.ch_name.args) == 0:
+                return hp
+            
+            # Currently only handle the case where all but the last index
+            # are constants.
+            assert all(isinstance(arg, AConst) for arg in hp.ch_name.args[:-1])
+
+            last_arg = hp.ch_name.args[-1]
+            new_name = hp.ch_name.name
+            for arg in hp.ch_name.args[:-1]:
+                if isinstance(arg.value, str):
+                    new_name = new_name + '_lb_' + arg.value + '_rb_'
+                else:
+                    new_name = new_name + '_l_' + str(arg) + '_r_'
+            if isinstance(last_arg, AConst):
+                if isinstance(last_arg.value, str):
+                    new_name = new_name + '_lb_' + last_arg.value + '_rb_'
+                else:
+                    new_name = new_name + '_l_' + str(last_arg) + '_r_'
+                new_args = ()
+                new_hp = OutputChannel(Channel(new_name, new_args), hp.expr)
+                return new_hp
+            elif isinstance(last_arg, AVar) and last_arg.name.startswith("_"):
+                raise NotImplementedError
+            else:
+                # Expression case
+                matches = find_match(hp.ch_name)
+                assert len(matches) > 0
+                if_hps = []
+                for match in sorted(matches):
+                    new_args = ()
+                    if isinstance(match.value, str):
+                        match_name = new_name + "_lb_" + match.value + "_rb_"
+                    else:
+                        match_name = new_name + "_l_" + str(match) + "_r_"
+                    if_hps.append((RelExpr("==", last_arg, match),
+                                   OutputChannel(Channel(match_name, new_args), hp.expr)))
+                return ITE(if_hps)
+
+        elif isinstance(hp, (SelectComm, ODE_Comm)):
+            new_io_comms = []
+            for comm_hp, out_hp in hp.io_comms:
+                if not isinstance(comm_hp, (InputChannel, OutputChannel)):
+                    raise TypeError
+                assert comm_hp.ch_name.name in concrete_map
+                if len(comm_hp.ch_name.args) == 0:
+                    new_io_comms.append((comm_hp, rec(out_hp)))
+                
+                assert all(isinstance(arg, AConst) for arg in comm_hp.ch_name.args[:-1])
+
+                last_arg = comm_hp.ch_name.args[-1]
+                new_name = comm_hp.ch_name.name
+                for arg in comm_hp.ch_name.args[:-1]:
+                    if isinstance(arg.value, str):
+                        new_name = new_name + '_lb_' + arg.value + '_rb_'
+                    else:
+                        new_name = new_name + '_l_' + str(arg) + '_r_'
+                if isinstance(last_arg, AConst):
+                    if isinstance(last_arg.value, str):
+                        new_name = new_name + '_lb_' + last_arg.value + '_rb_'
+                    else:
+                        new_name = new_name + '_l_' + str(last_arg) + '_r_'
+                    new_args = ()
+                    if isinstance(comm_hp, InputChannel):
+                        new_hp = InputChannel(Channel(new_name, new_args), comm_hp.var_name)
+                    else:
+                        new_hp = OutputChannel(Channel(new_name, new_args), comm_hp.expr)
+                    new_io_comms.append((new_hp, rec(out_hp)))
+                elif isinstance(last_arg, AVar) and last_arg.name.startswith("_"):
+                    # Wildcard case
+                    matches = find_match(comm_hp.ch_name)
+                    assert len(matches) > 0
+                    for match in sorted(matches):
+                        new_args = ()
+                        if isinstance(match.value, str):
+                            match_name = new_name + "_lb_" + match.value + "_rb_"
+                        else:
+                            match_name = new_name + "_l_" + str(match) + "_r_"
+                        if isinstance(comm_hp, InputChannel):
+                            new_io_comms.append((InputChannel(Channel(match_name, new_args), comm_hp.var_name),
+                                                 rec(out_hp.subst_comm({last_arg.name: match}))))
+                        else:
+                            new_io_comms.append((OutputChannel(Channel(match_name, new_args), comm_hp.expr),
+                                                 rec(out_hp.subst_comm({last_arg.name: match}))))
+                else:
+                    raise NotImplementedError
+            
+            if isinstance(hp, SelectComm):
+                return SelectComm(*new_io_comms)
+            else:
+                return ODE_Comm(hp.eqs, hp.constraint, new_io_comms)
+
+        elif isinstance(hp, (Var, Skip, Wait, Assign, Assert, Test, Log)):
+            return hp
+        elif isinstance(hp, Loop):
+            return Loop(rec(hp.hp), constraint=hp.constraint)
+        elif isinstance(hp, Sequence):
+            return Sequence(*(rec(sub_hp) for sub_hp in hp.hps))
+        elif isinstance(hp, ODE):
+            return ODE(hp.eqs, hp.constraint, out_hp=rec(hp.out_hp))
+        elif isinstance(hp, ITE):
+            new_if_hps = []
+            for cond, if_hp in hp.if_hps:
+                new_if_hps.append((cond, rec(if_hp)))
+            new_else_hp = None
+            if hp.else_hp is not None:
+                new_else_hp = rec(hp.else_hp)
+            return ITE(new_if_hps, new_else_hp)
+        else:
+            print(type(hp))
+            raise NotImplementedError
+
+    return rec(hp)
+
+def convert_infos_to_concrete_chs(infos: List[HCSPInfo]) -> List[HCSPInfo]:
+    concrete_chs = get_concrete_chs(infos)
+    new_infos = []
+    for info in infos:
+        new_hp = convert_to_concrete_chs(info.hp, concrete_chs)
+        new_procs = []
+        for proc in info.procedures:
+            new_procs.append(Procedure(proc.name, convert_to_concrete_chs(proc.hp, concrete_chs)))
+        new_infos.append(HCSPInfo(info.name, new_hp, outputs=info.outputs, procedures=new_procs))
+    return new_infos
+
+def has_all_concrete_chs(infos: List[HCSPInfo]) -> bool:
+    def rec(hp):
+        if isinstance(hp, (InputChannel, OutputChannel)):
+            if len(hp.ch_name.args) != 0:
+                print("Not empty chs:", hp)
+            if not all(isinstance(arg, AConst) for arg in hp.ch_name.args):
+                print("Non-concrete chs:", hp)
+                return False
+            else:
+                return True
+        elif isinstance(hp, (Var, Skip, Wait, Assign, Assert, Test, Log)):
+            return True
+        elif isinstance(hp, Loop):
+            return rec(hp.hp)
+        elif isinstance(hp, Sequence):
+            return all(rec(sub_hp) for sub_hp in hp.hps)
+        elif isinstance(hp, ODE):
+            return rec(hp.out_hp)
+        elif isinstance(hp, (ODE_Comm, SelectComm)):
+            return all(rec(comm) and rec(out_hp) for comm, out_hp in hp.io_comms)
+        elif isinstance(hp, ITE):
+            for _, if_hp in hp.if_hps:
+                if not rec(if_hp):
+                    return False
+            if hp.else_hp is not None and not rec(hp.else_hp):
+                return False
+            return True
+        else:
+            print(hp)
+            raise NotImplementedError
+
+    for info in infos:
+        if not rec(info.hp):
+            return False
+        for proc in info.procedures:
+            if not rec(proc.hp):
+                return False
+    return True
+
+def reduce_procedures(hp: HCSP, procs: Dict[str, HCSP], strict_protect:Optional[Set[str]] = None):
     """Reduce number of procedures in the process by inlining.
 
     hp : HCSP - input process.
@@ -1505,7 +1823,7 @@ def reduce_procedures(hp, procs, strict_protect=None):
 
     # First, construct the dependency relation. We use the empty string
     # to represent the toplevel process.
-    dep_relation = dict()
+    dep_relation: Dict[str, HCSP] = dict()
     dep_relation[""] = hp.get_contain_hps()
     for name, proc in procs.items():
         dep_relation[name] = proc.get_contain_hps()
