@@ -1,5 +1,5 @@
 from ss2hcsp.sl.sl_block import SL_Block
-from ss2hcsp.hcsp.expr import AVar, AConst, LogicExpr, RelExpr, IfExpr, OpExpr
+from ss2hcsp.hcsp.expr import AVar, AConst, RelExpr, IfExpr
 from ss2hcsp.hcsp import hcsp as hp
 
 
@@ -35,14 +35,3 @@ class Saturation(SL_Block):
         expr = self.get_expr()
         out_var = self.src_lines[0][0].name
         return {out_var: expr}
-
-    def get_var_map(self):
-        in_var = AVar(self.dest_lines[0].name)
-        cond0 = RelExpr("<", in_var, AConst(self.low_lim))
-        expr0 = AConst(self.low_lim)
-        cond1 = RelExpr(">", in_var, AConst(self.up_lim))
-        expr1 = AConst(self.up_lim)
-        cond2 = LogicExpr("&&", cond0.neg(), cond1.neg())
-        expr2 = in_var
-        out_var = self.src_lines[0][0].name
-        return {out_var: [(cond0, expr0), (cond1, expr1), (cond2, expr2)]}
