@@ -26,12 +26,3 @@ class Relation(SL_Block):
 
     def __repr__(self):
         return str(self)
-
-    def get_var_map(self):
-        in_vars = [AVar(line.name) for line in self.dest_lines]
-        cond0 = RelExpr(self.relation, in_vars[0], in_vars[1])
-        expr0 = AConst(1)
-        cond1 = cond0.neg()
-        expr1 = AConst(0)
-        out_var = self.src_lines[0][0].name
-        return {out_var: [(cond0, expr0), (cond1, expr1)]}
