@@ -36,8 +36,3 @@ class Bias(SL_Block):
         expr = self.get_expr()
         out_var = self.src_lines[0][0].name
         return {out_var: expr}
-
-    def get_var_map(self):
-        expr = self.get_expr()
-        out_var = self.src_lines[0][0].name
-        return {out_var: [(true_expr, expr)]}
