@@ -1,14 +1,15 @@
+"""Represents a single line in a Simulink diagram."""
+
 class SL_Line:
-    """Represents a single line in a Simulink diagram."""
-    def __init__(self, src, dest, src_port, dest_port, *, name="?", ch_name="?"):
+    def __init__(self, src: str, dest: str, src_port: int, dest_port: int, *, name="?", ch_name="?"):
         # Source and target block
         assert isinstance(src, str) and isinstance(dest, str)
-        self.src = src  # string
-        self.dest = dest  # string
+        self.src = src
+        self.dest = dest
 
         # Port number within source and target block
-        self.src_port = src_port  # nat
-        self.dest_port = dest_port  # nat
+        self.src_port = src_port
+        self.dest_port = dest_port
 
         self.name = name
         self.ch_name = ch_name
@@ -24,3 +25,9 @@ class SL_Line:
     def __repr__(self):
         return "SL_Line(%s, %s, %s, %s, %s, %s)" % \
             (self.src, self.dest, self.src_port, self.dest_port, self.name, self.ch_name)
+
+class UnknownLine(SL_Line):
+    def __init__(self):
+        super(UnknownLine, self).__init__("?", "?", 0, 0, name="??", ch_name="??")
+
+unknownLine = UnknownLine()
