@@ -56,47 +56,6 @@ class ExprTest(unittest.TestCase):
             expr = expr_parser.parse(s)
             self.assertEqual(str(expr), s)
 
-    def assertConditionalInst(self, test_data, res):
-        inst = expr.Conditional_Inst()
-        for var_name, cond_inst_str in test_data:
-            cond_inst = [(expr_parser.parse(cond), expr_parser.parse(inst))
-                         for cond, inst in cond_inst_str]
-            inst.add(var_name, cond_inst)
-
-        res_inst = dict()
-        for var_name, cond_inst_str in res:
-            cond_inst = [(expr_parser.parse(cond), expr_parser.parse(inst))
-                         for cond, inst in cond_inst_str]
-            res_inst[var_name] = cond_inst
-        self.assertEqual(inst.data, res_inst)
-
-    def testConditionalInst1(self):        
-        test_data = [
-            ("x", [("true", "a + 1")]),
-            ("y", [("a >= 1", "x"), ("a < 1", "x - 1")]), 
-        ]
-
-        res = [
-            ("x", [("true", "a + 1")]),
-            ("y", [("a >= 1", "a + 1"), ("a < 1", "(a + 1) - 1")])
-        ]
-
-        self.assertConditionalInst(test_data, res)
-
-    def testConditionalInst2(self):
-        test_data = [
-            ("w", [("in >= 5", "a"), ("in < 5", "b")]),
-            ("z", [("w >= 10", "x"), ("w < 10", "y")]),
-        ]
-
-        res = [
-            ("w", [("in >= 5", "a"), ("in < 5", "b")]),
-            ("z", [("a >= 10 && in >= 5 || b >= 10 && in < 5", "x"),
-                   ("a < 10 && in >= 5 || b < 10 && in < 5", "y")]),
-        ]
-
-        self.assertConditionalInst(test_data, res)
-
     def testParseHCSP(self):
         test_data = [
             "x1 := 3;",
